package example.localplugin.coolemall.computebox2;

import java.util.ArrayList;
import java.util.List;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.Core;
import schedframe.resources.computing.Node;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.manager.tasks.JobRegistryImpl;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;
import example.localplugin.BaseLocalSchedulingPlugin;
import gridsim.dcworms.DCWormsTags;

public class CB2_FCFS_LoadBalancing_SP extends BaseLocalSchedulingPlugin {

	public CB2_FCFS_LoadBalancing_SP () {
	}

	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			 ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();
		// choose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
			// our tasks are placed only in first queue (see
			// BaseLocalSchedulingPlugin.placeJobsInQueues() method)
			TaskQueue q = queues.get(0);
			
			// check all tasks in queue
			for (int i = 0; i < q.size(); i++) {
				TaskInterface<?> task = q.get(i);
				// if status of the tasks in READY
				if (task.getStatus() == DCWormsTags.READY) {

					String nodeName = chooseProvider(resourceManager, task);
					if (nodeName != null) {
						addToSchedulingPlan(plan, task, nodeName);
					}
				}
			}
			break;
		}
		return plan;
	}

	private String chooseProvider(ClusterResourceManager resourceManager, TaskInterface<?> task) {
		List<Node> nodes = resourceManager.getNodes();
		List<Node> avNodes = filterNodes(nodes, task);
		int nodeIdx = findLeastLoadedResourceIdx(avNodes);
		return nodeIdx != -1 ? avNodes.get(nodeIdx).getFullName() : null;
	}
	
	private List<Node> filterNodes(List<Node> nodes, TaskInterface<?> task){
		List<Node> filteredNodes = new ArrayList<Node>();
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 0;
		}
		for (Node node : nodes) {

			if (cpuRequest != 0) {

				List<Core> cores = node.getCores();
				if (cores.size() < cpuRequest) {
					if(cores.size() == 0){
						if(node.getProcessors().size() < cpuRequest)
							continue;
					}
				}

				int freeCores = 0;
				for(Core core: cores){
					if(core.getStatus() == ResourceStatus.FREE)
						freeCores++;
				}
				
				if(freeCores < cpuRequest)
					continue;
				
				filteredNodes.add(node);
			}
		}
		
		return filteredNodes;
	}
	
	private int findLeastLoadedResourceIdx(List<Node> nodes ){
		int resourceIdx = -1;
		int minLoad = Integer.MAX_VALUE;
		
		for(int i = 0; i < nodes.size(); i++){
			Node node = nodes.get(i);
			JobRegistry jr = new JobRegistryImpl(node.getFullName());
			int totalLoad = jr.getRunningTasks().size(); 
			if(totalLoad < minLoad){
				resourceIdx = i;
				minLoad = totalLoad;
			}
		}
		return resourceIdx;
	}

}

